package main

import (
	"context"
	"fmt"
	"log"
	"math/rand"
	"net/http"
	_ "net/http/pprof"
	"os"
	"sync"
	"time"

	"github.com/Shopify/sarama"
)

const letterBytes = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"

func RandStringBytes(n int) string {
	b := make([]byte, n)
	for i := range b {
		b[i] = letterBytes[rand.Intn(len(letterBytes))]
	}
	return string(b)
}

func main() {

	go func() {
		log.Println(http.ListenAndServe("localhost:6060", nil))
	}()

	sarama.Logger = log.New(os.Stdout, "[sarama] ", log.LstdFlags)

	topic := "my-test-hang-topic"

	producer := NewWrapSyncProducer(Config{
		Hosts: "localhost:9092,localhost:9093,localhost:9094",
		//Topic: topic,
	})
	cleanup := func() {
		if producer != nil {
			_ = producer.Close()
		}
	}
	defer cleanup()

	ticker := time.NewTicker(150 * time.Second)
	fmt.Println("current time:", time.Now())

	for {
		workerNum := 8
		wg := sync.WaitGroup{}
		wg.Add(workerNum)
		for i := 0; i < workerNum; i++ {
			go func(num int) {
				defer wg.Done()
				for j := 0; j < 100; j++ {
					producer.SendMessage(context.Background(), topic, RandStringBytes(32),
						fmt.Sprintf("testing %d at %v for routine %d", j, time.Now(), num))
					//time.Sleep(10 * time.Millisecond)
				}

			}(i)
		}
		wg.Wait()

		t := <-ticker.C
		fmt.Println("current time:", t)
	}

}
