package main

import (
	"context"
	"encoding/json"
	"hash/fnv"
	"strings"
	"time"

	"log"

	"github.com/Shopify/sarama"
)

type Config struct {
	// 集群配置以英文逗号隔开即可
	Hosts string `yaml:"hosts"`
	Topic string // 暂不从配置文件解析得到
}

type SyncProducer struct {
	sarama.SyncProducer
	version sarama.KafkaVersion
	//cfg     *config
}

// SendMessage calls sarama.SyncProducer.SendMessage and traces the request.
func (s *SyncProducer) SendMessage(ctx context.Context, topic string, key string, value interface{}) (partition int32, offset int64, err error) {
	valueBytes, err := json.Marshal(value)
	if err != nil {
		log.Printf("value数据格式化失败, value: %#v, err %#v\r\n", value, err)
		return 0, 0, err
	}
	msg := &sarama.ProducerMessage{
		Topic:     topic,
		Key:       sarama.StringEncoder(key),
		Value:     sarama.ByteEncoder(valueBytes),
		Timestamp: time.Now(),
	}

	partition, offset, err = s.SyncProducer.SendMessage(msg)
	return partition, offset, err
}

// SendMessages calls sarama.SyncProducer.SendMessages and traces the requests.
// TODO 待改造
func (s *SyncProducer) SendMessages(ctx context.Context, msgs []*sarama.ProducerMessage) error {
	// although there's only one call made to the SyncProducer, the saramaMessages are
	// treated individually, so we create a span for each one

	err := s.SyncProducer.SendMessages(msgs)
	return err
}

// wrapSyncProducer wraps a sarama.SyncProducer so that all produced saramaMessages
// are traced.
func wrapSyncProducer(saramaConfig *sarama.Config, producer sarama.SyncProducer) *SyncProducer {

	if saramaConfig == nil {
		saramaConfig = sarama.NewConfig()
	}
	return &SyncProducer{
		SyncProducer: producer,
		version:      saramaConfig.Version,
		//cfg:          cfg,
	}
}

func NewWrapSyncProducer(kafkaConfig Config) *SyncProducer {
	// TODO 定制化发送者配置入口 可以考虑做成函数选项
	cfg := sarama.NewConfig()
	// 发送完数据需要leader和follow都确认
	cfg.Producer.RequiredAcks = sarama.WaitForAll
	// Partition选择随机
	cfg.Producer.Partitioner = sarama.NewCustomHashPartitioner(fnv.New32a)
	// 成功交付的消息将在success channel返回
	cfg.Producer.Return.Successes = true
	cfg.Metadata.RefreshFrequency = 2 * time.Minute
	// 连接时间修改为3秒
	cfg.Net.DialTimeout = 3 * time.Second
	// 配置版本
	cfg.Version = sarama.DefaultVersion
	//_ = saramaprom.ExportMetrics(context.Background(), cfg.MetricRegistry, saramaprom.Options{})
	producer, err := sarama.NewSyncProducer(strings.Split(kafkaConfig.Hosts, ","), cfg)
	if err != nil {
		panic(err)
	}

	return wrapSyncProducer(cfg, producer)
}
